@@ -39,6 +39,7 @@ module RubiGen
       # Replay action manifest.  RewindBase subclass rewinds manifest.
       def invoke!
         manifest.replay(self)
+        after_generate
       end
 
       def dependency(generator_name, args, runtime_options = {})
@@ -55,7 +56,22 @@ module RubiGen
       def readme(*args)
       end
 
+      # Does nothing for all commands except Create.
+      def write_manifest
+      end
+      
       protected
+        def current_migration_number
+          Dir.glob("#{RAILS_ROOT}/#{@migration_directory}/[0-9]*_*.rb").inject(0) do |max, file_path|
+            n = File.basename(file_path).split('_', 2).first.to_i
+            if n > max then n else max end
+          end
+        end
+           
+        def next_migration_number
+          current_migration_number + 1
+        end
+             
         def migration_directory(relative_path)
           directory(@migration_directory = relative_path)
         end
@@ -68,19 +84,12 @@ module RubiGen
           not existing_migrations(file_name).empty?
         end
 
-        def current_migration_number
-          Dir.glob("#{@migration_directory}/[0-9]*.rb").inject(0) do |max, file_path|
-            n = File.basename(file_path).split('_', 2).first.to_i
-            if n > max then n else max end
-          end
-        end
-
-        def next_migration_number
-          current_migration_number + 1
-        end
-
         def next_migration_string(padding = 3)
-          "%.#{padding}d" % next_migration_number
+          if ActiveRecord::Base.timestamped_migrations
+            Time.now.utc.strftime("%Y%m%d%H%M%S")
+          else
+            "%.#{padding}d" % next_migration_number
+          end
         end
 
         def gsub_file(relative_destination, regexp, *args, &block)
@@ -92,29 +101,39 @@ module RubiGen
       private
         # Ask the user interactively whether to force collision.
         def force_file_collision?(destination, src, dst, file_options = {}, &block)
-          $stdout.print "overwrite #{destination}? [Ynaiqd] "
-          $stdout.flush
-          case $stdin.gets
-            when /d/i
+          stdout.print "overwrite #{destination}? (enter \"h\" for help) [Ynaiqd] "
+          stdout.flush
+          case $stdin.gets.chomp
+            when /\Ad\z/i
               Tempfile.open(File.basename(destination), File.dirname(dst)) do |temp|
                 temp.write render_file(src, file_options, &block)
                 temp.rewind
-                $stdout.puts `#{diff_cmd} #{dst} #{temp.path}`
+                stdout.puts `#{diff_cmd} #{dst} #{temp.path}`
               end
-              puts "retrying"
+              stdout.puts "retrying"
               raise 'retry diff'
-            when /a/i
-              $stdout.puts "forcing #{spec.name}"
+            when /\Aa\z/i
+              stdout.puts "forcing #{spec.name}"
               options[:collision] = :force
-            when /i/i
-              $stdout.puts "ignoring #{spec.name}"
+            when /\Ai\z/i
+              stdout.puts "ignoring #{spec.name}"
               options[:collision] = :skip
-            when /q/i
-              $stdout.puts "aborting #{spec.name}"
+            when /\Aq\z/i
+              stdout.puts "aborting #{spec.name}"
               raise SystemExit
-            when /n/i then :skip
-            when /y/i then :force
-            else force_file_collision?(destination, src, dst, file_options, &block)
+            when /\An\z/i then :skip
+            when /\Ay\z/i then :force
+            else
+              $stdout.puts <<-HELP.gsub(/^              /, '')
+              Y - yes, overwrite
+              n - no, do not overwrite
+              a - all, overwrite this and all others
+              i - ignore, skip any conflicts
+              q - quit, abort
+              d - diff, show the differences between the old and the new
+              h - help, show this help
+              HELP
+              raise 'retry'
           end
         rescue
           retry
@@ -201,7 +220,7 @@ module RubiGen
         # Determine full paths for source and destination files.
         source              = source_path(relative_source)
         destination         = destination_path(relative_destination)
-        destination_exists  = File.exists?(destination)
+        destination_exists  = File.exist?(destination)
 
         # If source and destination are identical then we're done.
         if destination_exists and identical?(source, destination, &block)
@@ -249,7 +268,7 @@ module RubiGen
           FileUtils.chmod(file_options[:chmod], destination)
         end
 
-        # Optionally add file to subversion
+        # Optionally add file to subversion or git
         system("svn add #{destination}") if options[:svn]
       end
 
@@ -267,6 +286,7 @@ module RubiGen
         files.each do |file_name|
           file "#{template_path}#{file_name}", "#{path}#{file_name}", options
         end
+        system("git add -v #{relative_destination}") if options[:git]
       end
 
       # Checks if the source and the destination file are identical. If
@@ -280,7 +300,7 @@ module RubiGen
       end
 
       # Generate a file using an ERuby template.
-      # Looks up and evalutes a template by name and writes the result.
+      # Looks up and evaluates a template by name and writes the result.
       #
       # The ERB template uses explicit trim mode to best control the
       # proliferation of whitespace in generated code.  <%- trims leading
@@ -320,22 +340,24 @@ module RubiGen
       end
 
       # Create a directory including any missing parent directories.
-      # Always directories which exist.
+      # Always skips directories which exist.
       def directory(relative_path)
         path = destination_path(relative_path)
-        if File.exists?(path)
+        if File.exist?(path)
           logger.exists relative_path
         else
           logger.create relative_path
           unless options[:pretend]
             FileUtils.mkdir_p(path)
+            # git doesn't require adding the paths, adding the files later will
+            # automatically do a path add.
 
             # Subversion doesn't do path adds, so we need to add
             # each directory individually.
             # So stack up the directory tree and add the paths to
             # subversion in order without recursion.
             if options[:svn]
-              stack=[relative_path]
+              stack = [relative_path]
               until File.dirname(stack.last) == stack.last # dirname('.') == '.'
                 stack.push File.dirname(stack.last)
               end
@@ -352,11 +374,10 @@ module RubiGen
       def readme(*relative_sources)
         relative_sources.flatten.each do |relative_source|
           logger.readme relative_source
-          puts File.read(source_path(relative_source)) unless options[:pretend]
+          stdout.puts File.read(source_path(relative_source)) unless options[:pretend]
         end
       end
 
-      # Display a README.
       def write_manifest(relative_destination)
         files = ([relative_destination] + Dir["#{destination_root}/**/*"])
         files.reject! { |file| File.directory?(file) }
@@ -392,7 +413,6 @@ module RubiGen
 
         # Optionally add file to subversion
         system("svn add #{destination}") if options[:svn]
-
       end
 
       # When creating a migration, it knows to find the first available file in db/migrate and use the migration.rb template.
@@ -436,18 +456,20 @@ module RubiGen
         # Raise a usage error with an informative WordNet suggestion.
         # Thanks to Florian Gross (flgr).
         def raise_class_collision(class_name)
-          message = <<end_message
-The name '#{class_name}' is reserved.
+          message = <<-end_message
+The name '#{class_name}' is either already used in your application or reserved.
 Please choose an alternative and run this generator again.
 end_message
           if suggest = find_synonyms(class_name)
-            message << "\n  Suggestions:  \n\n"
-            message << suggest.join("\n")
+            if suggest.any?
+              message << "\n  Suggestions:  \n\n"
+              message << suggest.join("\n")
+            end
           end
           raise UsageError, message
         end
 
-        SYNONYM_LOOKUP_URI = "http://wordnet.princeton.edu/cgi-bin/webwn2.0?stage=2&word=%s&posnumber=1&searchtypenumber=2&senses=&showglosses=1"
+        SYNONYM_LOOKUP_URI = "http://wordnet.princeton.edu/perl/webwn?s=%s"
 
         # Look up synonyms on WordNet.  Thanks to Florian Gross (flgr).
         def find_synonyms(word)
@@ -455,8 +477,8 @@ end_message
           require 'timeout'
           timeout(5) do
             open(SYNONYM_LOOKUP_URI % word) do |stream|
-              data = stream.read.gsub("&nbsp;", " ").gsub("<BR>", "")
-              data.scan(/^Sense \d+\n.+?\n\n/m)
+              # Grab words linked to dictionary entries as possible synonyms
+              data = stream.read.gsub("&nbsp;", " ").scan(/<a href="webwn.*?">([\w ]*?)<\/a>/s).uniq
             end
           end
         rescue Exception
@@ -471,7 +493,7 @@ end_message
       # Remove a file if it exists and is a file.
       def file(relative_source, relative_destination, file_options = {})
         destination = destination_path(relative_destination)
-        if File.exists?(destination)
+        if File.exist?(destination)
           logger.rm relative_destination
           unless options[:pretend]
             if options[:svn]
@@ -485,6 +507,19 @@ end_message
               # has no modifications so we can simply remove it
                 system("svn rm #{destination}")
               end
+            elsif options[:git]
+              if options[:git][:new][relative_destination]
+                # file has been added, but not committed
+                system("git reset HEAD #{relative_destination}")
+                FileUtils.rm(destination)
+              elsif options[:git][:modified][relative_destination]
+                # file is committed and modified
+                system("git rm -f #{relative_destination}")
+              else
+                # If the directory is not in the status list, it
+                # has no modifications so we can simply remove it
+                system("git rm #{relative_destination}")
+              end
             else
               FileUtils.rm(destination)
             end
@@ -506,7 +541,7 @@ end_message
         until parts.empty?
           partial = File.join(parts)
           path = destination_path(partial)
-          if File.exists?(path)
+          if File.exist?(path)
             if Dir[File.join(path, '*')].empty?
               logger.rmdir partial
               unless options[:pretend]
@@ -521,6 +556,8 @@ end_message
                   # has no modifications so we can simply remove it
                     system("svn rm #{path}")
                   end
+                # I don't think git needs to remove directories?..
+                # or maybe they have special consideration...
                 else
                   FileUtils.rmdir(path)
                 end
@@ -545,7 +582,7 @@ end_message
 
         migration_file_name = template_options[:migration_file_name] || file_name
         unless migration_exists?(migration_file_name)
-          puts "There is no migration named #{migration_file_name}"
+          stdout.puts "There is no migration named #{migration_file_name}"
           return
         end
 
